//
//
//
#include <tap.h>

//Every TAP application needs unique ID and TAP allocations that have same ID cant execute at same time.
//ID consists of DWORD( 4byte ) and the MSB bit should be set by 1.
//The following is how to use in source codes.
#define ID_USERTAP 0xbdb00c01
TAP_ID( ID_USERTAP );

//Every TAP application has to include Program name, Author name, and Program description as following
//manners in source code.
TAP_PROGRAM_NAME("epg_schedule v0.01");
TAP_AUTHOR_NAME("bdb");
TAP_DESCRIPTION("enables epg schedule for other muxes");
TAP_ETCINFO(__DATE__);

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//
//tsr_commander stuff
//
#include "TSRCommander.inc"

extern void TSRCommanderConfigDialog();
extern bool TSRCommanderExitTAP();
extern int is_tsr_commander_running(void);

//----------------------------------------------------------------------------
//
//main
//

int TAP_Main(void);
dword TAP_EventHandler(word event, dword param1, dword param2);
int exTAP_EIT_Schedule(int enable);
int exTAP_EIT_NowNext(int enable);

#define pSysID 0xa3fffffa
extern int _appl_version;
char *tap_name;
dword fw_tap_table = 0;

//----------------------------------------------------------------------------
//
//tsr_commander stuff
//
typedef struct
{
    byte signature[32];					//TFAMIPS
    dword id;
    byte padding1[28];
    const char name[MAX_PROGRAM_NAME];
    const char authorName[MAX_AUTHOR_NAME];
    const char description[MAX_DESCRIPTION];
    const char etcStr[MAX_ETCINFO_STR];
} type_TAPHeader;

typedef struct
{
    // From libtap.a
    dword _initData;
    void *TAP_Main;
    dword _tap_gotStart;
    dword _tap_gotEnd;
    dword _tap_textEnd;
    dword _tap_fixupStart;
    dword _tap_fixupEnd;
    void *TAP_EventHandler;
    dword _gp;
    dword _end;

    void *pTAP_SystemProc;
    void *_appl_version;
    void *spare0;
    void *pTAP_GetState;
    void *pTAP_ExitNormal;
    //etc
} type_TAPEntryPoints;

typedef struct
{
    dword firmwaregp;
    dword tapgp;
    dword unknown1;                         //tap malloc base address, before + 7ffe, & 0xFFFF8000
    type_TAPHeader *header;                 //after & 0xffff8000
    type_TAPEntryPoints *entryPoints;
    dword loadAddress;
    dword unknown2;                         //this taps current directory ?
    dword zero;
} type_TAPProcess;

bool TSRCommanderExitTAP()
{
    exTAP_EIT_Schedule(0);
    TSRCommanderErase();
    TAP_Print("\nClosing %s", tap_name);
    TAP_Exit();
    return FALSE;
}

void TSRCommanderConfigDialog()
{
}

int is_tsr_commander_running(void)
{
    int i;
    type_TAPProcess *TAPProcess;

    for (i=0; i<16; i++) {
        TAPProcess = (type_TAPProcess *)(fw_tap_table + (i*32));
        if (TAPProcess->firmwaregp) {
            if (TAPProcess->header) {
                if (TAPProcess->header->id == 0x807a10b2) {  //tsr_commander
                    return 1;
                }
            }
        }
    }
    return 0;
}

//----------------------------------------------------------------------------
//
//main
//
int TAP_Main(void)
{
    word SysID = *(word*)pSysID;

    tap_name =  (char *)__tap_program_name__;
    TAP_Print("\nOpening %s", tap_name);

    //determine support for different firmwares
    switch(SysID) {
        case 456:
            break;
        default:
            TAP_Print("\nUnsupported system id - %d", SysID);
            TAP_Print("\n");
            return -1;
    }
    switch(_appl_version) {
        case 0x1225:
            fw_tap_table = 0x8017187c;
            break;
        default:
            TAP_Print("\nUnsupported firmware version - %04x", _appl_version);
            TAP_Print("\n");
            return -1;
    }

    TSRCommanderInit(0, FALSE);

    //patch the eit schedule for 'other' muxes
    exTAP_EIT_Schedule(1);

    //patch the eit nownext for 'other' muxes
    exTAP_EIT_NowNext(1);

    return 1;                 //0=application mode, 1=tsr mode
}

dword TAP_EventHandler(word event, dword param1, dword param2)
{
    switch (event) {
        case EVT_IDLE:
            TSRCommanderWork();
            break;
    }
    return param1;
}

//----------------------------------------------------------------------------
//
//eit schedule patch
//
//this patch changes the eit filter/mask to enable the eit schedule information
//for the other muxes, as well as the currrent mux.  It also removes the service_id
//filter to enusre more rapid filling of the EPG.
//this patch should be used in conjunction with enlarging the EPG database to ensure
//that there is space for 5000 entries.
//
//

int exTAP_EIT_Schedule(int enable)
{
    if (enable) {
        TAP_Print("\nApplying EIT Schedule Patch");
        if (*(dword *)0x80181fbc == 0x3407f050) *(dword *)0x80181fbc = 0x3407ce50;
        if (*(dword *)0x80181e90 == 0x8fb60078) *(dword *)0x80181e90 = 0x3416ffff;
    }
    else {
        TAP_Print("\nRemoving EIT Schedule Patch");
        if (*(dword *)0x80181fbc == 0x3407ce50) *(dword *)0x80181fbc = 0x3407f050;
        if (*(dword *)0x80181e90 == 0x3416ffff) *(dword *)0x80181e90 = 0x8fb60078;
    }
}

//----------------------------------------------------------------------------
//
//eit nownext patch
//
//this patch changes the eit filter/mask to enable the eit nownext information
//for the other muxes, as well as the currrent mux.  It also removes the service_id
//filter to ensure more rapid filling of the EPG.
//
//

int exTAP_EIT_NowNext(int enable)
{
    if (enable) {
        TAP_Print("\nApplying EIT NowNext Patch");
        if (*(dword *)0x80181950 == 0x8fb60078) *(dword *)0x80181950 = 0x3416ffff;         //filter_00: service_id=ffff
        if (*(dword *)0x80181c5c == 0x8fb60080) *(dword *)0x80181c5c = 0x3416ffff;         //filter_01: service_id=ffff
        if (*(dword *)0x80182078 == 0x3407ff4e) *(dword *)0x80182078 = 0x3407ff4e;         //change 4e -> 4e,4f
        if (*(dword *)0x80182094 == 0x3407ff4e) *(dword *)0x80182094 = 0x3407ff4e;         //change 4e -> 4e,4f
    }
    else {
        TAP_Print("\nRemoving EIT NowNext Patch");
        if (*(dword *)0x80181950 == 0x3416ffff) *(dword *)0x80181950 = 0x8fb60078;
        if (*(dword *)0x80181c5c == 0x3416ffff) *(dword *)0x80181c5c = 0x8fb60080;
        if (*(dword *)0x80182078 == 0x3407ff4e) *(dword *)0x80182078 = 0x3407ff4e;
        if (*(dword *)0x80182094 == 0x3407ff4e) *(dword *)0x80182094 = 0x3407ff4e;
    }
}


//----------------------------------------------------------------------------
